@@ -53,6 +53,7 @@ ConfigWidget::ConfigWidget(QObject *parent, const KPluginMetaData &data, const Q
     connect(m_ui.drawTitleBarSeparator, &QAbstractButton::clicked, this, &ConfigWidget::updateChanged);
     connect(m_ui.roundedCorners, &QAbstractButton::clicked, this, &ConfigWidget::updateChanged);
     connect(m_ui.otherCornerRadius, SIGNAL(valueChanged(int)), SLOT(updateChanged()));
+    connect(m_ui.floatingTitlebar, &QAbstractButton::clicked, this, &ConfigWidget::updateChanged);
 
     // track animations changes
     connect(m_ui.animationsEnabled, &QAbstractButton::clicked, this, &ConfigWidget::updateChanged);
@@ -85,6 +86,7 @@ void ConfigWidget::load()
     m_ui.drawTitleBarSeparator->setChecked(m_internalSettings->drawTitleBarSeparator());
     m_ui.roundedCorners->setChecked(m_internalSettings->roundedCorners());
     m_ui.otherCornerRadius->setValue(m_internalSettings->otherCornerRadius());
+    m_ui.floatingTitlebar->setChecked(m_internalSettings->floatingTitlebar());
 
     // load shadows
     if (m_internalSettings->shadowSize() <= InternalSettings::ShadowVeryLarge)
@@ -120,6 +122,7 @@ void ConfigWidget::save()
     m_internalSettings->setDrawTitleBarSeparator(m_ui.drawTitleBarSeparator->isChecked());
     m_internalSettings->setRoundedCorners(m_ui.roundedCorners->isChecked());
     m_internalSettings->setOtherCornerRadius(m_ui.otherCornerRadius->value());
+    m_internalSettings->setFloatingTitlebar(m_ui.floatingTitlebar->isChecked());
 
     m_internalSettings->setShadowSize(m_ui.shadowSize->currentIndex());
     m_internalSettings->setShadowStrength(qRound(qreal(m_ui.shadowStrength->value() * 255) / 100));
@@ -167,6 +170,7 @@ void ConfigWidget::defaults()
     m_ui.drawTitleBarSeparator->setChecked(m_internalSettings->drawTitleBarSeparator());
     m_ui.roundedCorners->setChecked(m_internalSettings->roundedCorners());
     m_ui.otherCornerRadius->setValue(m_internalSettings->otherCornerRadius());
+    m_ui.floatingTitlebar->setChecked(m_internalSettings->floatingTitlebar());
 
     m_ui.shadowSize->setCurrentIndex(m_internalSettings->shadowSize());
     m_ui.shadowStrength->setValue(qRound(qreal(m_internalSettings->shadowStrength() * 100) / 255));
@@ -199,6 +203,8 @@ void ConfigWidget::updateChanged()
         modified = true;
     else if (m_ui.otherCornerRadius->value() != m_internalSettings->otherCornerRadius())
         modified = true;
+    else if (m_ui.floatingTitlebar->isChecked() != m_internalSettings->floatingTitlebar())
+        modified = true;
 
     // animations
     else if (m_ui.animationsEnabled->isChecked() != m_internalSettings->animationsEnabled())
